<?php
/**
 * i-doIT
 *
 * DAO: Global category for memory
 *
 * @package i-doit
 * @subpackage CMDB_Categories
 * @author Niclas Potthast <npotthast@i-doit.org> - 2005-08-04
 * @copyright Copyright 2004, 2005 - synetics GmbH
 * @license http://dev.perl.org/perl6/rfc/346.html Artistic License 2.0
 */

class isys_cmdb_dao_category_g_memory extends isys_cmdb_dao_category_global {
	
	protected $m_filter = 
 		array(	"isys_catg_memory_list__title",
 				"isys_catg_memory_list__description",
 				"isys_catg_memory_list__capacity", 				
 				"isys_memory_title__title",
 				"isys_memory_title__description",
 				"isys_memory_type__title",
 				"isys_memory_type__description",
 				"isys_memory_manufacturer__title",
 				"isys_memory_manufacturer__description",
 				"isys_memory_unit__title",
 				"isys_memory_unit__description"
 				);
	
	
	public function get_category_id() {
		return C__CATG__MEMORY;
	}
	
	
	/**
	 * Get possible filter rows
	 *
	 * @return array
	 */
	public function get_filter() {
		return $this->m_filter;
	}
	
	
	/**
	 * Return Data by Object-ID
	 *
	 * @param int $p_obj_id
	 * @param string $p_condition
	 * 
	 * @return isys_component_dao_result
	 */
	public function get_data_by_object($p_obj_id, $p_condition = null, $p_status = null) {
		return $this->get_data(null, $p_obj_id, $p_condition, null, $p_status);
	}
	
	
	/**
	 * Return Category Data
	 *
	 * @param [int $p_id]h
	 * @param [int $p_obj_id]
	 * @param [string $p_condition]
	 * 
	 * @return isys_component_dao_result
	 */
	public function get_data($p_catg_list_id=null, $p_obj_id=null, $p_condition="", $p_filter=null, $p_status=null) {
		$p_condition .= $this->prepare_filter($p_filter);
		
		$l_sql = "SELECT * FROM isys_catg_distributor ".
					"LEFT JOIN isys_catg_memory_list ".
						"ON ".
						"isys_catg_memory_list__isys_catg_memory__id = ".
						"isys_catg_distributor__isys_catg_memory__id ".
					"LEFT JOIN isys_memory_title ".
						"ON ".
						"isys_memory_title__id = ".
						"isys_catg_memory_list__isys_memory_title__id ".
					"LEFT JOIN isys_memory_manufacturer ".
						"ON ".
						"isys_memory_manufacturer__id = ".
						"isys_catg_memory_list__isys_memory_manufacturer__id ".
					"LEFT JOIN isys_memory_type ".
						"ON ".
						"isys_memory_type__id = ".
						"isys_catg_memory_list__isys_memory_type__id ".
					"LEFT JOIN isys_memory_unit ".
						"ON ".
						"isys_memory_unit__id = ".
						"isys_catg_memory_list__isys_memory_unit__id ".
					"LEFT JOIN isys_catg_memory ".
						"ON ".
						"isys_catg_memory__id = ".
						"isys_catg_memory_list__isys_catg_memory__id ".
						
					"WHERE 1 ";
		
		$l_sql .= $p_condition;
		
		if (!empty($p_obj_id)) {
			$l_sql .= " AND (isys_catg_distributor__isys_obj__id = '{$p_obj_id}')";
		}
		
		if (!empty($p_catg_list_id)) {
			$l_sql .= " AND (isys_catg_memory_list__id = '{$p_catg_list_id}')";
		}

		if (!empty($p_status)) {
			$l_sql .= " AND (isys_catg_memory_list__status = '{$p_status}')";
		}
		
		return $this->retrieve($l_sql . ";");
	}
	

	/**
	 * Import-Handler for category memory
	 *
	 * @author Dennis St�cken <dstuecken@i-doit.org>
	 */
	public function import($p_data) {
		$l_status	= -1;
		$l_cat 		= -1;
		$l_arID     = array();
		
		if (is_array($p_data)) {
			
			foreach ($p_data as $l_data) {
				
				$l_list_id = 0;
				$this->create_element($l_cat, $l_list_id);
				
				$_POST['C__CMDB__CAT__COMMENTARY'] = "";
				
				/* Memory Title */
				if (empty($l_data["title"])) {
					$l_title = $l_data["form"];
					
					if (strlen(($l_data["bank"])) > 1) {
						$l_title .= " (" . $l_data["bank"] . ")";
					}
				} else {
					$l_title = $l_data["title"];
				}
				
				$_POST['C__CATG__MEMORY_TITLE_ID'] = isys_import::check_dialog(	"isys_memory_title", $l_title);
				
				/* Manufacturer */
				$_POST['C__CATG__MEMORY_MANUFACTURER'] = isys_import::check_dialog("isys_memory_manufacturer", (empty($l_data["manufacturer"])) ? "n/a" : $l_data["manufacturer"]);
				
				/* Unit */
				if (empty($l_data["unit"])) {
					$l_unit = "MB";
				} else {
					$l_unit = $l_data["unit"];
				}
				
				$_POST['C__CATG__MEMORY_UNIT'] = isys_import::check_dialog(	"isys_memory_unit", $l_unit);
				
				$_POST['C__CATG__MEMORY_TYPE'] = isys_import::check_dialog(	"isys_memory_type", $l_data["type"]);
								
				/* Capacity */
				if (empty($l_data["capacity"])) $l_capacity = $l_data["size"];
				  else $l_capacity = $l_data["capacity"];
				
				$_POST['C__CATG__MEMORY_CAPACITY'] = $l_capacity;
				$_POST["C__CATG__MEMORY_QUANTITY"] = 1;
				
				/* Save */
				$l_arID[] = $this->save_element($l_cat, $l_status, $l_list_id);
			}
		}
		
		return true;
	}
	
	/**
	 * Syncronize category data
	 *
	 * @param array $p_data
	 * @param int $p_object_id
	 * @return bool
	 */
	public function sync($p_data, $p_object_id) {
		
		foreach ($p_data as $data) {
			$l_catdata = $this->get_data($data->get_attribute("data_id"), $p_object_id)->__to_array();
		
			isys_import_handler_cmdb::sync_category($l_catdata, $data, $this->get_category_id());
			
			$l_list_id = $l_catdata["isys_catg_memory_list__id"];
			
			if (empty($l_list_id)) {
				$ok = $this->create($l_catdata["isys_catg_distributor__isys_catg_memory__id"], 
									C__RECORD_STATUS__NORMAL,
									$l_catdata["isys_catg_memory_list__isys_memory_title__id"],
									$l_catdata["isys_catg_memory_list__isys_memory_manufacturer__id"],
									$l_catdata["isys_catg_memory_list__isys_memory_type__id"],
									$l_catdata["isys_catg_memory_list__isys_memory_unit__id"],
									$l_catdata["isys_catg_memory_list__capacity"],
									$l_catdata["isys_catg_memory_list__description"]);
			} else {
				$ok =  $this->save($l_list_id,
								   C__RECORD_STATUS__NORMAL,
								   $l_catdata["isys_catg_memory_list__isys_memory_manufacturer__id"],
								   $l_catdata["isys_catg_memory_list__isys_memory_title__id"],
								   $l_catdata["isys_catg_memory_list__isys_memory_type__id"],
								   $l_catdata["isys_catg_memory_list__isys_memory_unit__id"],
								   $l_catdata["isys_catg_memory_list__capacity"],
								   $l_catdata["isys_catg_memory_list__description"]);
			}
			
			if ($ok)
				continue;
			else
				return false;
		}
		return $ok;
	}
	
	
	/**
	 * Executes the query to save the category entry given by its ID $p_cat_level
	 *
	 * @param int $p_cat_level
	 * @param int $p_newRecStatus
	 * @param int $p_manufacturerID
	 * @param int $p_titleID
	 * @param int $p_typeID
	 * @param int $p_unitID
	 * @param int $p_capacity	 
	 * @param String $p_description
	 * @return boolean true, if transaction executed successfully, else false
	 * @author Dennis Bluemer <dbluemer@i-doit.org>
	 */
	public function save($p_cat_level, $p_newRecStatus, $p_manufacturerID, $p_titleID, $p_typeID, $p_unitID, $p_capacity, $p_description) {
		$l_strSql = "UPDATE isys_catg_memory_list SET ".
					"isys_catg_memory_list__description = ".$this->convert_sql_text($p_description).", ".
					"isys_catg_memory_list__isys_memory_title__id = ".$this->convert_sql_id($p_titleID).", ".
					"isys_catg_memory_list__isys_memory_manufacturer__id  = ".$this->convert_sql_id($p_manufacturerID).", ".
					"isys_catg_memory_list__isys_memory_type__id  = ".$this->convert_sql_id($p_typeID).", ".
					"isys_catg_memory_list__isys_memory_unit__id  = ".$this->convert_sql_id($p_unitID).", ".
					"isys_catg_memory_list__capacity  = ".$this->convert_sql_id($p_capacity).", ".					
					"isys_catg_memory_list__status = ".$this->convert_sql_id($p_newRecStatus)." ".
					"WHERE isys_catg_memory_list__id = ".$this->convert_sql_id($p_cat_level);
		
		if ($this->update($l_strSql)) {
			return $this->apply_update();
		}
		else
			return false;
	}
	

	/**
	 * Save global category menory element
	 * 
	 * @param $p_cat_level level to save, default 0
	 * @param &$p_intOldRecStatus __status of record before update
	 * @author Oliver Steigleder <osteigleder@i-doit.org> - 20051026
	 */
	public function save_element(&$p_cat_level, &$p_intOldRecStatus, $p_id = null) {
		$intErrorCode = -1;
		$l_bNewItem   = false;
		
		if (isys_glob_get_param(C__CMDB__GET__CATLEVEL) == 0 &&
			isys_glob_get_param(C__CMDB__GET__CATG) == C__CATG__OVERVIEW &&
			isys_glob_get_param(C__GET__NAVMODE) == C__NAVMODE__SAVE) {
			$l_bNewItem = true;
		}
		
		if (isys_glob_get_param(C__CMDB__GET__CATLEVEL) == -1) {
			$l_bNewItem = true;
		}
		
		$l_catdata = $this->get_result()->__to_array();
		
		$p_intOldRecStatus = $l_catdata["isys_catg_memory_list__status"];
		
		
		if ($l_bNewItem) {
			// New item, so create entries
			
			// Overview page and no input was given
			if (isys_glob_get_param(C__CMDB__GET__CATG) == C__CATG__OVERVIEW &&
				$_POST['C__CATG__MEMORY_MANUFACTURER']  == -1 &&
				$_POST['C__CATG__MEMORY_TITLE_ID']      == -1 &&
				$_POST['C__CATG__MEMORY_TYPE']		    == -1 &&
				$_POST['C__CATG__MEMORY_UNIT']		    == -1 &&
				$_POST['C__CATG__MEMORY_CAPACITY']	    == "") return null;
			
			$l_quantity = $_POST["C__CATG__MEMORY_QUANTITY"];
			
			if (!is_numeric($l_quantity) || $l_quantity < 1) 
				throw new isys_exception_dao("Invalid value for quantity");
				
			$i=1;
			while($i++ <= $l_quantity) {
				$l_id = $this->create($l_catdata["isys_catg_memory__id"],
									  C__RECORD_STATUS__NORMAL,
									  $_POST['C__CATG__MEMORY_TITLE_ID'],
									  $_POST['C__CATG__MEMORY_MANUFACTURER'],									  
									  $_POST['C__CATG__MEMORY_TYPE'],
									  $_POST['C__CATG__MEMORY_UNIT'],
									  $_POST['C__CATG__MEMORY_CAPACITY'],
									  $_POST['C__CMDB__CAT__COMMENTARY']);
									  
				if ($l_id != false) {
					$this->m_strLogbookSQL = $this->get_last_query();
				}
			}
			$p_cat_level = 1;
			return $l_id;
		}
		else {
			// existing entry, so update this
			
			if (!empty($p_id)) {
				$l_catdata['isys_catg_memory_list__id'] = $p_id;
			}
			
			if ($l_catdata['isys_catg_memory_list__id'] != "")	{
				
				$l_bRet = $this->save($l_catdata["isys_catg_memory_list__id"],
									  C__RECORD_STATUS__NORMAL,
									  $_POST['C__CATG__MEMORY_TITLE_ID'],
									  $_POST['C__CATG__MEMORY_MANUFACTURER'],									  
									  $_POST['C__CATG__MEMORY_TYPE'],
									  $_POST['C__CATG__MEMORY_UNIT'],
									  $_POST['C__CATG__MEMORY_CAPACITY'],								  
									  $_POST['C__CMDB__CAT__COMMENTARY']);
									  
				$this->m_strLogbookSQL = $this->get_last_query();
	
			}
			else if ($l_catdata['isys_catg_memory_list__id'] > 0) {
				if ($l_catdata['isys_catg_memory_list__status'] == C__RECORD_STATUS__BIRTH) {
					$this->delete($l_catdata["isys_catg_memory_list__id"]);
				}
			}
		}
		return $l_bRet == true ? null : $intErrorCode;
	}
	
	
	public function delete($p_list_id) {
		
		$l_sql = "DELETE FROM isys_catg_memory_list WHERE ".
					"(isys_catg_memory_list__id = '".$p_list_id."');";
					
		return $this->update($l_sql) && $this->apply_update();
	}
	
	
	/**
	 * Save global category memory element
	 * 
	 * @param $p_cat_level level to save, default 0
	 * @param &$p_new_id returns the __id of the new record
	 * @author Oliver Steigleder <osteigleder@i-doit.org> - 20051026
	 */
	public function create_element($p_cat_level, &$p_new_id) {
		$p_new_id = -1;
		return null;
	}
	
	
	/**
	 * Executes the query to create the category entry referenced by isys_catg_memory__id $p_fk_id
	 *
	 * @param int $p_fk_id
	 * @param int $p_newRecStatus
	 * @param int $p_titleID
	 * @param int $p_manufacturerID
	 * @param int $p_typeID
	 * @param int $p_unitID
	 * @param int $p_capacity
	 * @param String $p_description
	 * @return int the newly created ID or false
	 * @author Dennis Bluemer <dbluemer@i-doit.org>
	 */
	public function create($p_fk_id, $p_newRecStatus, $p_titleID, $p_manufacturerID, $p_typeID, $p_unitID, $p_capacity, $p_description) {
		$l_strSql = "INSERT INTO isys_catg_memory_list SET ".
					"isys_catg_memory_list__description = ".$this->convert_sql_text($p_description).", ".
					"isys_catg_memory_list__isys_memory_manufacturer__id  = ".$this->convert_sql_id($p_manufacturerID).", ".
					"isys_catg_memory_list__isys_memory_title__id = ".$this->convert_sql_id($p_titleID).", ".
					"isys_catg_memory_list__isys_memory_type__id  = ".$this->convert_sql_id($p_typeID).", ".
					"isys_catg_memory_list__isys_memory_unit__id  = ".$this->convert_sql_id($p_unitID).", ".
					"isys_catg_memory_list__capacity  = ".$this->convert_sql_id($p_capacity).", ".					
					"isys_catg_memory_list__status = ".$this->convert_sql_id($p_newRecStatus).", ".
					"isys_catg_memory_list__isys_catg_memory__id = ".$this->convert_sql_id($p_fk_id);
		
		if ($this->update($l_strSql) && $this->apply_update()) {
			return $this->get_last_insert_id();
		}
		else
			return false;
	}
	

	/**
	 * @author Oliver Steigleder <osteigleder@i-doit.org> - 2005-10-26
	 * @version Niclas Potthast <npotthast@i-doit.org> - 2006-11-24
 	 * @return bool
 	 */
	public function is_validate_post_data() {
		global $g_comp_template_language_manager;

		$l_retValid = true;
		$l_arrTomAdditional = array();

 		$this->set_additional_rules($l_retValid == false ? $l_arrTomAdditional : null);
		$this->set_validation($l_retValid);

 		return $l_retValid;
	}
	

	public function &get_ui() {
		global $g_comp_template;

		return new isys_cmdb_ui_category_g_memory($g_comp_template);
	}
	

	public function __construct(isys_component_database &$p_db)	{
		parent::__construct($p_db);
	}
}
?>